// Example Java Code for the lecture on Graph Theory in DM534 by Daniel Merkle Nov. 8, 2018.
// For simplicity, no Exception Handling is implemented.

public class MyMatrix {

    // Assume M and N are both square (size x size) matrices 
    public static int[][] multiplySquared(int[][] M, int[][] N) {

        int size = M.length;

	// In Java, all values are initialized to 0
	// for good measure, the initialization is kept here
        int[][] C = new int[size][size];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
		C[i][j] = 0;
            }
        }

        for (int i = 0; i < size; i++) { 
            for (int j = 0; j < size; j++) { 
                for (int k = 0; k < size; k++) { 
                    C[i][j] += M[i][k] * N[k][j];
                }
            }
        }

        return C;
    }

    // Assume two matrices M and N, not necessarily squared
    // (not needed further on in the lecture) 
    public static int[][] multiplyGeneral(int[][] M, int[][] N) {

        int mRows = M.length;
        int mColumns = M[0].length;
        int nRows = N.length;
        int nColumns = N[0].length;

	// In Java, all values are initialized to 0
	// for good measure, the initialization is kept here
        int[][] C = new int[mRows][nColumns];
        for (int i = 0; i < mRows; i++) {
            for (int j = 0; j < nColumns; j++) {
		C[i][j] = 0;
            }
        }

        for (int i = 0; i < mRows; i++) { // mRow
            for (int j = 0; j < nColumns; j++) { // nColumn
                for (int k = 0; k < mColumns; k++) { 
                    C[i][j] += M[i][k] * N[k][j];
                }
            }
        }

        return C;
    }

    public static void print(int[][] M) {
	for (int i = 0; i < M.length; i++) {
	    for (int j = 0; j < M[0].length; j++)
                System.out.print(M[i][j] + " ");
            System.out.println();
        }
    }

    public static void main(String[] args) {
	int[][] M = {{ 1, 0, 2, 3},
		     {-1, 2, 2, 1}};
	
	int[][] N = {{ 1, 2, 3},
		     { 4, 5, 2},
		     { 1, 2, 1},
		     { 1, 2, 5}};
	
	int[][] S = {{ 1, 2, 0},
		     { 2, 0, 1},
		     {-1, 2, 3}};
	
        MyMatrix matrix = new MyMatrix();
	System.out.println("\nInitial Matrix M:");
	print(M);

	System.out.println("\nInitial Matrix N:");
	print(N);

	System.out.println("\nM x N:");
	print(multiplyGeneral(M, N));

	System.out.println("\nS x S:");
	print(multiplySquared(S, S));
	
    }
}
